import { html, css, LitElement } from "lit";
import { property, customElement } from "lit/decorators.js";

@customElement("arx-header")
export class Header extends LitElement {
  @property({ type: String }) override title = "Eve";
  @property({ type: String }) url = "eve://home";

  @property({ type: Boolean }) canGoBack = false;
  @property({ type: Boolean }) canGoForward = false;

  private searchQuery = "";
  private searchInput: HTMLInputElement | null = null;

  static override styles = [
    css`
      header {
        background: rgba(255, 255, 255, 0.5);
        backdrop-filter: blur(10px);
        border: 1px solid rgba(255, 255, 255, 0.6);
        box-shadow: 0 8px 32px rgba(0, 0, 0, 0.2);
        z-index: 999999;
        background: var(--primary);
        height: var(--font-2xl);
        font-size: var(--header-height);
        transition: all 0.3s ease;
        display: flex;
        align-items: center;
        padding: 0 var(--space-md);
      }

      .nav-buttons {
        display: flex;
        gap: var(--space-xs);
        padding-right: var(--space-xs);

        button {
          text-decoration: none;
          color: var(--light);
          background: rgba(255, 255, 255, 0.1);
          border: 1px solid rgba(255, 255, 255, 0.2);
          padding: var(--space-xs);
          border-radius: 100%;
          font-size: var(--font-md);
          backdrop-filter: blur(10px);
          display: flex;
          align-items: center;
          justify-content: center;
          transition: all 0.2s ease;

          &:hover {
            background: rgba(255, 255, 255, 0.2);
          }

          &.disabled {
            opacity: 0.5;
            pointer-events: none;
          }
        }
      }

      .search-container {
        flex: 1;
        position: relative;

        input {
          background: rgba(255, 255, 255, 0.5);
          backdrop-filter: blur(10px);
          border: 1px solid rgba(255, 255, 255, 0.6);
          box-shadow: 0 8px 32px rgba(0, 0, 0, 0.2);
          color: var(--light);
          width: 100%;
          text-align: center;
          cursor: text;
          font-size: 0.9rem;
          width: 100%;
          padding: var(--space-xs);
          text-align: center;

          &:not(:focus)::placeholder {
            color: var(--light);
            opacity: 1;
          }

          &:focus {
            background: rgba(0, 0, 0, 0.2);
          }
        }
      }
    `,
  ];

  override render() {
    return html`
      <header>
        <div class="nav-buttons">
          <button
            class=${this.canGoBack ? "" : "disabled"}
            @click=${() => this.dispatchEvent(new CustomEvent("go-back"))}
            aria-label="Go back"
          >
            <iconify-icon icon="material-symbols:arrow-back"></iconify-icon>
          </button>
          <button
            class=${this.canGoForward ? "" : "disabled"}
            @click=${() => this.dispatchEvent(new CustomEvent("go-forward"))}
            aria-label="Go forward"
          >
            <iconify-icon icon="material-symbols:arrow-forward"></iconify-icon>
          </button>
        </div>
        <div class="search-container" @click=${this.focusSearch}>
          <input
            ref=${this.searchInput}
            type="text"
            .value=${this.searchQuery}
            @keyup=${this.handleSearch}
          />
        </div>
      </header>
    `;
  }

  private focusSearch() {
    this.searchInput?.focus();
  }

  private handleSearch(e: KeyboardEvent) {
    if (e.key !== "Enter") return;
    const hash = (e.target as HTMLInputElement).value.replace("eve://", "#");
    window.location.hash = hash;
  }
}
