import { LitElement, html, css } from "lit";
import { customElement, state } from "lit/decorators.js";
import { animate } from "@lit-labs/motion";
import * as nostrTools from "@nostr/tools/pure";
import * as nip06 from "@nostr/tools/nip06";
import * as nip19 from "@nostr/tools/nip19";
import * as nip49 from "@nostr/tools/nip49";
import { ndk, setSigner } from "@/ndk";
import { NDKEvent, NDKKind, NDKPrivateKeySigner } from "@nostr-dev-kit/ndk";
import { encodeBase64 } from "@std/encoding/base64";
import { randomBytes } from "@noble/ciphers/webcrypto";

@customElement("arx-initial-setup")
export class InitialSetup extends LitElement {
  @state() private currentPage = 1;
  @state() private isAnimating = false;
  @state() private seedPhrase = "";
  @state() private userName = "";
  @state() private profileImage = "";
  @state() private lightningAddress = "";

  get encryptionPassphrase() {
    let encryptionPassphrase = localStorage.getItem("encryption_key");
    if (!encryptionPassphrase) {
      encryptionPassphrase = encodeBase64(randomBytes(32));
      localStorage.setItem("encryption_key", encryptionPassphrase);
    }
    return encryptionPassphrase;
  }

  static override styles = css`
    :host {
      display: block;
      width: 100%;
      --primary-color: var(--eve-primary-color, #4a90e2);
      --primary-hover: var(--eve-primary-hover, #357abd);
      --secondary-color: var(--eve-secondary-color, #6c757d);
      --secondary-hover: var(--eve-secondary-hover, #5a6268);
      --text-color: var(--eve-text-color, #2c3e50);
      --text-secondary: var(--eve-text-secondary, #64748b);
      --background-color: var(--eve-background-color, #ffffff);
      --error-color: var(--eve-error-color, #dc3545);
      --success-color: var(--eve-success-color, #28a745);
      --spacing-unit: 0.25rem;
      --border-radius: 8px;
      --transition-fast: 150ms cubic-bezier(0.4, 0, 0.2, 1);
      --transition-normal: 300ms cubic-bezier(0.4, 0, 0.2, 1);
      --shadow-sm: 0 1px 2px 0 rgb(0 0 0 / 0.05);
      --shadow-md: 0 4px 6px -1px rgb(0 0 0 / 0.1);

      font-family: system-ui, -apple-system, sans-serif;
      -webkit-font-smoothing: antialiased;
      color: var(--text-color);
    }

    .welcome-container {
      max-width: min(800px, 90vw);
      margin: 0 auto;
      padding: calc(var(--spacing-unit) * 8);
      animation: fadeIn var(--transition-normal);
    }

    section {
      margin-bottom: calc(var(--spacing-unit) * 12);
    }

    .alpha-badge {
      display: inline-flex;
      align-items: center;
      background: var(--primary-color);
      color: white;
      padding: 0.15em 0.5em;
      border-radius: 999px;
      font-size: 0.75em;
      font-weight: 500;
      text-transform: uppercase;
      letter-spacing: 0.05em;
    }

    h1,
    h2,
    h3 {
      margin: 0;
      line-height: 1.2;
    }

    h1 {
      font-size: clamp(2rem, 5vw, 2.5rem);
      font-weight: 700;
      background: linear-gradient(
        45deg,
        var(--primary-color),
        var(--primary-hover)
      );
      -webkit-background-clip: text;
      color: transparent;
      margin-bottom: calc(var(--spacing-unit) * 8);
    }

    h2 {
      font-size: clamp(1.5rem, 3vw, 1.75rem);
      font-weight: 600;
      margin-bottom: calc(var(--spacing-unit) * 4);
    }

    h3 {
      font-size: 1.25rem;
      font-weight: 600;
      margin-bottom: calc(var(--spacing-unit) * 4);
    }

    p {
      margin: 0 0 calc(var(--spacing-unit) * 4);
      line-height: 1.6;
      color: var(--text-secondary);
    }

    .input-group {
      display: flex;
      gap: calc(var(--spacing-unit) * 4);
      margin-top: calc(var(--spacing-unit) * 6);
    }

    input {
      flex: 1;
      padding: calc(var(--spacing-unit) * 4);
      border: 2px solid var(--text-secondary);
      border-radius: var(--border-radius);
      font-size: 1rem;
      transition: all var(--transition-fast);
    }

    input:focus {
      outline: none;
      border-color: var(--primary-color);
      box-shadow: 0 0 0 3px rgba(74, 144, 226, 0.2);
    }

    .button {
      position: relative;
      display: inline-flex;
      align-items: center;
      gap: calc(var(--spacing-unit) * 2);
      padding: calc(var(--spacing-unit) * 3) calc(var(--spacing-unit) * 6);
      border: none;
      border-radius: var(--border-radius);
      font-weight: 500;
      cursor: pointer;
      transition: all var(--transition-fast);
      &:disabled {
        opacity: 0.5;
        cursor: not-allowed;
      }
    }

    .button.primary {
      background: linear-gradient(
        45deg,
        var(--primary-color),
        var(--primary-hover)
      );
      color: white;
      box-shadow: var(--shadow-md);
    }

    .button.secondary {
      background: linear-gradient(
        45deg,
        var(--secondary-color),
        var(--secondary-hover)
      );
      color: white;
    }

    .button:hover {
      transform: translateY(-2px);
    }

    .button:active {
      transform: translateY(0);
    }

    .navigation {
      display: flex;
      justify-content: space-between;
      margin-top: calc(var(--spacing-unit) * 8);
    }

    @keyframes fadeIn {
      from {
        opacity: 0;
        transform: translateY(20px);
      }
      to {
        opacity: 1;
        transform: translateY(0);
      }
    }

    @media (max-width: 640px) {
      .welcome-container {
        padding: calc(var(--spacing-unit) * 4);
      }

      .input-group {
        flex-direction: column;
      }

      .button {
        width: 100%;
        justify-content: center;
      }
    }

    pre {
      white-space: normal;
    }
    code {
      white-space: pre;
    }

    .note {
      display: block;
      color: #666;
      font-size: 0.875rem;
      margin-top: 0.5rem;
      padding-left: 0.5rem;
      border-left: 2px solid #ddd;
      font-style: italic;
      line-height: 1.4;
      max-width: 600px;
      opacity: 0.9;
    }

    .note:hover {
      opacity: 1;
      border-left-color: #999;
    }

    fieldset {
      border: 2px solid #3498db;
      border-radius: 8px;
      padding: 20px;
      margin: 20px 0;
      background-color: #f8f9fa;
      box-shadow: 0 2px 4px rgba(0, 0, 0, 0.1);
    }

    legend {
      padding: 0 10px;
      background-color: #3498db;
      color: white;
      font-weight: bold;
      border-radius: 4px;
      font-size: 1.1em;
    }

    fieldset:hover {
      border-color: #2980b9;
      box-shadow: 0 4px 8px rgba(0, 0, 0, 0.15);
      transition: all 0.3s ease;
    }

    .external-link {
      color: #2970ff;
      text-decoration: none;
      font-weight: 500;
      transition: color 0.2s ease;
    }

    .external-link:hover {
      color: #1a56db;
      text-decoration: underline;
    }

    .external-link:after {
      content: "↗";
      display: inline-block;
      margin-left: 0.25rem;
      font-size: 0.875rem;
    }
  `;

  private handleNavigation(page: number) {
    if (this.isAnimating) return;
    this.isAnimating = true;
    this.currentPage = page;
    setTimeout(() => (this.isAnimating = false), 300);
  }

  private onSeedPhraseInput(event: Event) {
    this.seedPhrase = (event.target as HTMLInputElement).value;
  }

  private generateSeedPhrase() {
    this.seedPhrase = nip06.generateSeedWords();
  }

  private isValidSeedPhrase() {
    const words = this.seedPhrase.split(" ");
    if (words.length !== 12) return false;
    if (!nip06.validateWords(words.join(" "))) return false;
    return true;
  }

  private renderPageOne() {
    return html`
      <main class="welcome-container" ${animate()}>
        <section>
          <h1>Welcome to Eve</h1>
          <h2>Your Private Community Network</h2>
          <p>
            Connect, share, and engage with your community in a secure,
            members-only space designed just for you.
          </p>
        </section>

        <section>
          <div class="alpha-badge">
            <iconify-icon icon="mdi:alpha"></iconify-icon> Alpha Preview
          </div>
          <p>
            We're actively developing Eve to create the best possible
            experience. Your feedback helps shape our platform.
          </p>
        </section>

        <section>
          <div class="action-buttons">
            <a href="https://arx-ccn.com/eve-feedback" class="button primary">
              <iconify-icon icon="mdi:feedback"></iconify-icon>Share Feedback
            </a>
            <a
              href="https://arx-ccn.com/report-eve-bug"
              class="button secondary"
            >
              <iconify-icon icon="mdi:bug"></iconify-icon>Report a Bug
            </a>
          </div>
        </section>

        <div class="navigation">
          <span></span>
          <button
            @click=${() => this.handleNavigation(2)}
            class="button primary"
          >
            Next
          </button>
        </div>
      </main>
    `;
  }

  private renderPageTwo() {
    return html`
      <main class="welcome-container" ${animate()}>
        <section>
          <h1>Getting Started</h1>
          <h2>Creating a Community</h2>
          <p>
            Connect with others by joining an existing community or creating
            your own.
          </p>
          <p>
            During this alpha phase, community setup requires a few manual
            steps. We're actively working to streamline this process in future
            updates.
          </p>
        </section>

        <section>
          <h3>Seed Phrase</h3>
          <p>
            Enter your community's seed phrase below or generate a new one to
            create a community.
          </p>
          <p>
            <b>Important</b>: Keep your seed phrase secure. Anyone with access
            to it can join your community.
          </p>
          <p>
            In an upcoming release, we'll implement MLS (Messaging Layer
            Security) to enable simpler invitation-based community access, as
            well as improve your community's security.
          </p>
          <div class="input-group">
            <input
              @input=${this.onSeedPhraseInput}
              .value=${this.seedPhrase}
              id="seed-input"
              type="text"
              placeholder="Enter seed phrase..."
            />
            <button
              @click=${() => this.generateSeedPhrase()}
              class="button secondary"
            >
              Generate
            </button>
          </div>
        </section>

        <div class="navigation">
          <button
            @click=${() => this.handleNavigation(1)}
            class="button secondary"
          >
            Back
          </button>
          <button
            @click=${() => this.handleNavigation(3)}
            ?disabled=${!this.isValidSeedPhrase()}
            class="button primary"
          >
            Continue
          </button>
        </div>
      </main>
    `;
  }

  private async startRelay() {
    await window.relay.writeSeed(this.seedPhrase);
    await window.relay.start(this.encryptionPassphrase);
  }

  private renderPageThree() {
    return html`
      <main class="welcome-container" ${animate()}>
        <section>
          <h2>Configure Eve Relay</h2>
          <p>
            During this alpha phase, manual relay configuration is required.
            This process will be automated in future releases.
          </p>
          <p>Please press the button below to start the relay.</p>
          <button @click=${() => this.startRelay()} class="button primary">
            Start Relay
          </button>
          <p>
            Having trouble? Our team is here to help if you encounter any
            issues.
          </p>
          <p>Click Continue once the relay is running.</p>
        </section>

        <div class="navigation">
          <button
            @click=${() => this.handleNavigation(2)}
            class="button secondary"
          >
            Back
          </button>
          <button
            @click=${() => this.handleNavigation(4)}
            class="button primary"
          >
            Continue
          </button>
        </div>
      </main>
    `;
  }

  private onUserNameInput(e: Event) {
    this.userName = (e.target as HTMLInputElement).value;
  }

  private onProfileImageInput(e: Event) {
    this.profileImage = (e.target as HTMLInputElement).value;
  }

  private onLightningAddressInput(e: Event) {
    this.lightningAddress = (e.target as HTMLInputElement).value;
  }

  private renderPageFour() {
    return html`
      <main class="welcome-container" ${animate()}>
        <section>
          <h2>Complete Your Profile</h2>
          <p>Great progress! Let's set up your community profile.</p>
          <p>
            Your profile information will be encrypted and visible only to
            community members.
          </p>
          <div class="profile-form">
            <fieldset>
              <legend>Display Name</legend>
              <input
                id="username"
                type="text"
                .value=${this.userName}
                @input=${this.onUserNameInput}
                placeholder="Enter your name"
              />
            </fieldset>
            <fieldset>
              <legend>Profile Picture</legend>
              <input
                id="profile-image"
                type="text"
                .value=${this.profileImage}
                @input=${this.onProfileImageInput}
                placeholder="Enter image URL"
              />
              <small class="note">
                Direct file uploads will be supported in a future update. For
                now, please provide an image URL or leave blank.
              </small>
            </fieldset>
          </div>
        </section>
        <div class="navigation">
          <button
            @click=${() => this.handleNavigation(3)}
            class="button secondary"
          >
            Back
          </button>
          <button
            @click=${() => this.handleNavigation(5)}
            class="button primary"
          >
            Final Step
          </button>
        </div>
      </main>
    `;
  }

  private renderPageFive() {
    return html`
      <main class="welcome-container" ${animate()}>
        <section>
          <h2>Payment Setup</h2>
          <p>Almost done! Let's set up your payment options.</p>
          <p>
            Enter your existing Lightning address below for payments within the
            community. If you don't have one, leave this field blank and we'll
            automatically generate one for you through
            <a target="_blank" href="https://npub.cash" class="external-link"
              >npub.cash</a
            >.
          </p>
          <input
            id="lightning-address"
            type="text"
            .value=${this.lightningAddress}
            @input=${this.onLightningAddressInput}
            placeholder="your@lightning.address"
          />
          <small class="note">
            Your Lightning address enables secure, instant payments within the
            community.
          </small>
        </section>

        <div class="navigation">
          <button
            @click=${() => this.handleNavigation(4)}
            class="button secondary"
          >
            Back
          </button>
          <button @click=${() => this.goToFinalStep()} class="button primary">
            Next
          </button>
        </div>
      </main>
    `;
  }

  private async goToFinalStep() {
    const randomPrivateKey = nostrTools.generateSecretKey();
    const encryptedNsec = nip49.encrypt(
      randomPrivateKey,
      this.encryptionPassphrase
    );
    const npub = nip19.npubEncode(nostrTools.getPublicKey(randomPrivateKey));

    if (!this.lightningAddress) this.lightningAddress = `${npub}@npub.cash`;

    localStorage.setItem("ncryptsec", encryptedNsec);

    setSigner(new NDKPrivateKeySigner(randomPrivateKey));
    await ndk.connect(5000);

    const event = new NDKEvent(ndk);
    event.kind = NDKKind.Metadata;
    event.content = JSON.stringify({
      name: this.userName,
      image: this.profileImage || undefined,
      lud16: this.lightningAddress,
    });
    await event.sign();
    await event.publish();

    this.handleNavigation(6);
  }

  private renderPageSix() {
    return html`
      <main class="welcome-container" ${animate()}>
        <section>
          <h2>Done!</h2>
          <p>
            That's it! You're all set to start using the community. We hope you
            enjoy it!
          </p>
          <p>Your community's seed phrase is: <b>${this.seedPhrase}</b></p>
          <p>
            Please store this seed phrase somewhere safe and secure. It will be
            required to invite new members to the community.
          </p>
          <p>Your username is: <b>${this.userName}</b></p>
          ${this.profileImage
            ? html` <p>
                Your profile image is: <img .src=${this.profileImage} />
              </p>`
            : ""}
          <p>Your lightning address is: <b>${this.lightningAddress}</b></p>
        </section>

        <div class="navigation">
          <button
            @click=${() => this.handleNavigation(5)}
            class="button secondary"
          >
            Back
          </button>
          <button @click=${this.finish} class="button primary">Finish</button>
        </div>
      </main>
    `;
  }

  finish() {
    this.dispatchEvent(
      new CustomEvent("finish", {
        detail: {
          userName: this.userName,
          profileImage: this.profileImage,
          lightningAddress: this.lightningAddress,
        },
        bubbles: true,
        composed: true,
      })
    );
  }

  override render() {
    switch (this.currentPage) {
      case 1:
        return this.renderPageOne();
      case 2:
        return this.renderPageTwo();
      case 3:
        return this.renderPageThree();
      case 4:
        return this.renderPageFour();
      case 5:
        return this.renderPageFive();
      case 6:
        return this.renderPageSix();
      default:
        return html`<div class="welcome-container">Loading...</div>`;
    }
  }
}
