import { LitElement, html, css } from "lit";
import { customElement, property, state } from "lit/decorators.js";
import { getSigner, ndk } from "@/ndk";
import { NDKEvent } from "@nostr-dev-kit/ndk";

@customElement("arx-phora-topic-creator")
export class PhoraTopicCreator extends LitElement {
  @property({ type: String })
  categoryId = "";

  @state()
  private newTopic = "";

  @state()
  private topicContent = "";

  @state()
  private isCreating = false;

  static override styles = css`
    :host {
      display: block;
    }

    input,
    textarea {
      width: 100%;
      margin-bottom: 1rem;
      padding: 0.5rem;
      border: 1px solid #ccc;
      border-radius: 4px;
    }

    textarea {
      min-height: 200px;
      resize: vertical;
    }

    .error {
      color: red;
      margin-bottom: 1rem;
    }

    .category-id {
      color: #666;
      margin-bottom: 1rem;
      font-family: monospace;
    }
  `;

  private async doCreateTopic() {
    if (this.isCreating) return;

    if (this.newTopic.length < 3) {
      alert("Topic title must be at least 3 characters long");
      return;
    }

    if (this.topicContent.length < 10) {
      alert("Topic content must be at least 10 characters long");
      return;
    }

    this.isCreating = true;

    try {
      await getSigner();
      const event = new NDKEvent(ndk);
      event.kind = 11;
      event.tags = [
        ["subject", this.newTopic],
        ["e", this.categoryId],
      ];
      event.content = this.topicContent;
      await event.sign();
      await event.publish();

      this.dispatchEvent(
        new CustomEvent("topic-created", {
          bubbles: true,
          composed: true,
          detail: {
            topicId: event.id,
            categoryId: this.categoryId,
          },
        })
      );

      this.newTopic = "";
      this.topicContent = "";
    } catch (error) {
      console.error("Failed to create topic:", error);
      alert("Failed to create topic");
    } finally {
      this.isCreating = false;
    }
  }

  private handleTopicInput(e: InputEvent) {
    this.newTopic = (e.target as HTMLInputElement).value;
  }

  private handleContentInput(e: InputEvent) {
    this.topicContent = (e.target as HTMLTextAreaElement).value;
  }

  override render() {
    return html`
      <div class="category-id">Category ID: ${this.categoryId}</div>

      <input
        type="text"
        placeholder="New Topic"
        .value=${this.newTopic}
        @input=${this.handleTopicInput}
        ?disabled=${this.isCreating}
      />

      <textarea
        placeholder="Topic. You can use Markdown here."
        .value=${this.topicContent}
        @input=${this.handleContentInput}
        ?disabled=${this.isCreating}
      ></textarea>

      <div class="button-group">
        <arx-phora-button
          @click=${() => this.dispatchEvent(new CustomEvent("cancel"))}
          ?disabled=${this.isCreating}
        >
          Cancel
        </arx-phora-button>

        <arx-phora-button
          @click=${this.doCreateTopic}
          ?disabled=${this.isCreating}
        >
          ${this.isCreating ? "Creating..." : "Create"}
        </arx-phora-button>
      </div>
    `;
  }
}
