import { LitElement, html, css } from "lit";
import { customElement, property, state } from "lit/decorators.js";
import { getSigner, ndk } from "@/ndk";
import type { NDKSubscription } from "@nostr-dev-kit/ndk";

import "@components/Breadcrumbs";
import "@components/ForumPost";
import "@components/PhoraButton";

interface ForumPost {
  id: string;
  npub: string;
  date: Date;
  content: string;
}

@customElement("arx-phora-topic-view")
export class PhoraTopicView extends LitElement {
  @property({ type: String })
  topicId = "";

  @state()
  override title = "";

  @state()
  private posts: ForumPost[] = [];

  private subscription: NDKSubscription | undefined;

  static override styles = css`
    :host {
      display: block;
    }

    .topic {
      max-width: 1200px;
      padding: 1em;
      background: rgba(255, 255, 255, 0.8);
      border-radius: 0.5rem;
      display: grid;
      gap: 1rem;
      grid-template-columns: [column-1] auto [column-2] 1fr;
    }

    .header {
      background: var(--primary);
      color: oklch(100% 0 0);
      padding: 1rem 1.5rem;
      border-radius: 0.5rem 0.5rem 0 0;
      font-weight: 500;
      display: flex;
      justify-content: space-between;
      align-items: center;
    }

    .posts-container {
      display: flex;
      flex-direction: column;
      gap: 1rem;
    }

    .actions {
      margin-top: 1rem;
    }
  `;

  override async connectedCallback() {
    super.connectedCallback();
    await this.loadTopic();
  }

  override disconnectedCallback() {
    super.disconnectedCallback();
    if (this.subscription) {
      this.subscription.stop();
    }
  }

  private async loadTopic() {
    try {
      await getSigner();
      const event = await ndk.fetchEvent(this.topicId);

      if (!event) {
        throw new Error("Could not load topic");
      }

      this.title = event.tags.find((tag) => tag[0] === "subject")?.[1] || "";

      this.posts = [
        {
          id: event.id,
          npub: event.pubkey,
          date: new Date((event.created_at || 0) * 1000),
          content: event.content,
        },
      ];

      // Subscribe to new posts
      this.subscription = ndk
        .subscribe({
          kinds: [1111],
          "#e": [this.topicId],
        })
        .on("event", (event) => {
          this.posts = [
            ...this.posts,
            {
              id: event.id,
              npub: event.pubkey,
              date: new Date((event.created_at || 0) * 1000),
              content: event.content,
            },
          ];
        });
    } catch (error) {
      console.error("Failed to load topic:", error);
      alert("Could not load topic");
    }
  }

  override render() {
    const breadcrumbItems = [
      { text: "Home", href: "/" },
      { text: "Phora", href: "/phora" },
      { text: this.title },
    ];

    return html`
      <arx-breadcrumbs .items=${breadcrumbItems}></arx-breadcrumbs>

      <div class="header">
        <span>${this.title}</span>
      </div>

      <div class="posts-container">
        ${this.posts.map(
          (post) => html`
            <arx-forum-post
              class="topic"
              id=${post.id}
              .topicId=${this.topicId}
              .npub=${post.npub}
              .date=${post.date}
              .content=${post.content}
            ></arx-forum-post>
          `
        )}
      </div>

      <div class="actions">
        <arx-phora-button href="/phora/new-post/${this.topicId}">
          New Post
        </arx-phora-button>
      </div>
    `;
  }
}
