import { LitElement, html, css } from "lit";
import { customElement, property, state } from "lit/decorators.js";
import { when } from "lit/directives/when.js";
import type { NDKUserProfile } from "@nostr-dev-kit/ndk";
import { getUserProfile } from "../ndk";

@customElement("arx-profile-route")
export class NostrProfile extends LitElement {
  @property({ type: String })
  npub = "";

  @state()
  private profile: NDKUserProfile | undefined;

  @state()
  private error: string | null = null;

  static override styles = css`
    :host {
      display: block;
    }

    .banner-container {
      position: relative;
      height: 20rem;
      overflow: hidden;
    }

    .banner-image {
      position: absolute;
      inset: 0;
      transform: scale(1);
      transition: transform 700ms;
    }

    .banner-image:hover {
      transform: scale(1.05);
    }

    .banner-image img {
      width: 100%;
      height: 100%;
      object-fit: cover;
    }

    .banner-overlay {
      position: absolute;
      inset: 0;
      background: linear-gradient(
        to bottom,
        transparent,
        rgba(0, 0, 0, 0.2),
        rgba(0, 0, 0, 0.4)
      );
    }

    .profile-container {
      max-width: 64rem;
      margin: 0 auto;
      padding: 0 1rem;
      position: relative;
      z-index: 10;
    }

    .profile-container.with-banner {
      margin-top: -8rem;
    }

    .profile-container.no-banner {
      margin-top: 4rem;
    }

    .profile-card {
      background-color: rgba(255, 255, 255, 0.95);
      border-radius: 0.75rem;
      box-shadow: 0 4px 6px rgba(0, 0, 0, 0.1);
      padding: 1.5rem;
      backdrop-filter: blur(8px);
    }

    .profile-content {
      display: flex;
      flex-direction: column;
      gap: 1.5rem;
    }

    @media (min-width: 768px) {
      .profile-content {
        flex-direction: row;
        align-items: flex-start;
      }
    }

    .profile-image-container {
      position: relative;
    }

    .profile-image {
      width: 10rem;
      height: 10rem;
      border-radius: 50%;
      object-fit: cover;
      border: 4px solid white;
      box-shadow: 0 2px 4px rgba(0, 0, 0, 0.1);
      transition: transform 300ms;
    }

    .profile-image:hover {
      transform: scale(1.05);
    }

    .profile-image-placeholder {
      width: 10rem;
      height: 10rem;
      border-radius: 50%;
      background: linear-gradient(to bottom right, #e5e7eb, #d1d5db);
      display: flex;
      align-items: center;
      justify-content: center;
    }

    .placeholder-icon {
      width: 5rem;
      height: 5rem;
      color: #9ca3af;
    }

    .profile-info {
      flex: 1;
    }

    .profile-header {
      display: flex;
      flex-direction: column;
      justify-content: space-between;
    }

    @media (min-width: 768px) {
      .profile-header {
        flex-direction: row;
        align-items: center;
      }
    }

    .display-name {
      font-size: 1.875rem;
      font-weight: bold;
      color: #111827;
      display: flex;
      align-items: center;
      gap: 0.5rem;
    }

    .verified-icon {
      color: #3b82f6;
    }

    .nip05 {
      color: #4b5563;
      display: flex;
      align-items: center;
      gap: 0.25rem;
      margin-top: 0.25rem;
    }

    .action-buttons {
      display: flex;
      gap: 0.75rem;
      margin-top: 1rem;
    }

    @media (min-width: 768px) {
      .action-buttons {
        margin-top: 0;
      }
    }

    .follow-button {
      padding: 0.5rem 1.5rem;
      border-radius: 9999px;
      font-weight: 500;
      transition: all 300ms;
      background-color: #3b82f6;
      color: white;
    }

    .follow-button:hover {
      background-color: #2563eb;
    }

    .follow-button.following {
      background-color: #e5e7eb;
      color: #1f2937;
    }

    .follow-button.following:hover {
      background-color: #d1d5db;
    }

    .copy-button {
      padding: 0.5rem;
      border-radius: 9999px;
      background-color: #f3f4f6;
      transition: background-color 300ms;
    }

    .copy-button:hover {
      background-color: #e5e7eb;
    }

    .copy-icon {
      width: 1.25rem;
      height: 1.25rem;
    }

    .links-section {
      display: grid;
      grid-auto-columns: minmax(300px, 1fr);
      gap: 1.5rem;
      margin-top: 2rem;
      padding-top: 1.5rem;
      border-top: 1px solid #e5e7eb;
    }

    .link-item {
      display: flex;
      align-items: center;
      gap: 0.5rem;
      padding: 1rem;
      border-radius: 0.5rem;
      background-color: #f9fafb;
      transition: background-color 300ms;
    }

    .link-item:hover {
      background-color: #f3f4f6;
    }

    .link-icon {
      width: 1.5rem;
      height: 1.5rem;
    }

    .link-icon.website {
      color: #3b82f6;
    }

    .link-icon.lightning {
      color: #eab308;
    }

    .bio {
      white-space: pre-line;
      font-size: 0.9rem;
      color: #4b5563;
      margin: 1rem 0;
      padding-top: 1rem;
      line-height: 1.5;
    }

    .animate-gradient {
      background-size: 200% 200%;
      animation: gradient 15s ease infinite;
    }

    @keyframes gradient {
      0% {
        background-position: 0% 50%;
      }

      50% {
        background-position: 100% 50%;
      }

      100% {
        background-position: 0% 50%;
      }
    }
  `;

  protected override async firstUpdated() {
    try {
      this.profile = await getUserProfile(this.npub);
    } catch (err) {
      this.error = "Failed to load profile";
      console.error(err);
    }
  }

  private get displayName() {
    if (!this.profile) return this.npub;
    return (
      this.profile.displayName || this.profile.name || this.npub.substring(0, 8)
    );
  }

  override render() {
    if (this.error) {
      return html`<arx-error-view .error=${this.error}></arx-error-view>`;
    }

    if (!this.profile) {
      return html`<arx-loading-view></arx-loading-view>`;
    }

    return html`
      ${when(
        this.profile.banner,
        () => html`
          <div class="banner-container">
            <div class="banner-image">
              <img src=${this.profile!.banner} alt="Banner" />
            </div>
            <div class="banner-overlay"></div>
          </div>
        `
      )}

      <div
        class=${this.profile.banner
          ? "profile-container with-banner"
          : "profile-container no-banner"}
      >
        <div class="profile-card">
          <div class="profile-content">
            <div class="profile-image-container">
              ${when(
                this.profile.image,
                () =>
                  html`<img
                    src=${this.profile!.image}
                    alt="Profile"
                    class="profile-image"
                  />`,
                () => html`
                  <div class="profile-image-placeholder">
                    <svg-icon
                      icon="mdi:account"
                      class="placeholder-icon"
                    ></svg-icon>
                  </div>
                `
              )}
            </div>

            <div class="profile-info">
              <div class="profile-header">
                <div>
                  <h1 class="display-name">
                    ${this.displayName}
                    ${when(
                      this.profile.verified,
                      () => html`
                        <span class="verified-icon">
                          <svg-icon icon="mdi:check-decagram"></svg-icon>
                        </span>
                      `
                    )}
                  </h1>
                  ${when(
                    this.profile.nip05,
                    () => html`
                      <p class="nip05">
                        <svg-icon icon="mdi:at"></svg-icon>
                        ${this.profile!.nip05}
                      </p>
                    `
                  )}
                </div>
              </div>

              ${when(
                this.profile.about,
                () => html` <p class="bio">${this.profile!.about}</p> `
              )}
            </div>
          </div>

          <div class="links-section">
            ${when(
              this.profile.website,
              () => html`
                <a
                  href=${this.profile!.website}
                  target="_blank"
                  class="link-item"
                >
                  <svg-icon icon="mdi:web" class="link-icon website"></svg-icon>
                  <span>${this.profile!.website}</span>
                </a>
              `
            )}
            ${when(
              this.profile.lud16,
              () => html`
                <a href="lightning:${this.profile!.lud16}" class="link-item">
                  <svg-icon
                    icon="mdi:lightning-bolt"
                    class="link-icon lightning"
                  ></svg-icon>
                  <span>${this.profile!.lud16}</span>
                </a>
              `
            )}
          </div>
        </div>
      </div>
    `;
  }
}
